@@ -1,10 +1,15 @@
 import codecs
+import itertools
 import os
+import re
 import sys
 import subprocess
 import zipfile
 
-import Image
+try:
+    import Image
+except ImportError:
+    from PIL import Image
 
 from xml.etree import cElementTree as et
 
@@ -50,6 +55,26 @@ parser.add_option('--download-source-link', dest='download_source_link',
     action='store_true', default=False)
 
 
+invalid_filename_characters_re = re.compile(
+        '[^-\w]')
+
+filenames = set()
+
+
+def make_filename(text, unique=True):
+    fname = text.encode('ascii', 'replace').decode()
+    fname = invalid_filename_characters_re.sub('_', fname)
+    if unique:
+        if fname in filenames:
+            for n in itertools.count(1):
+                nfname = '%s%s' % (fname, n)
+                if nfname not in filenames:
+                    fname = nfname
+                    break
+        filenames.add(fname)
+    return fname
+
+
 class RstFile(object):
     headings = ['=', '-', '~', "^", '"', "'"]
 
@@ -391,8 +416,7 @@ class SphinxWriter(object):
         if self.current_file == self.files['index']:
             self.current_file.directive('toctree', maxdepth=2)
         self.current_file = RstFile()
-        fname = text.replace(' ', '').replace('/', '').replace(':', '_')
-        fname = fname.encode('ascii', 'ignore').decode()
+        fname = make_filename(text)
         self.files[fname] = self.current_file
 
         self.files['index'].append(fname)
@@ -696,6 +720,8 @@ class ODTReader(object):
     def is_list_numbered(self, node):
         style_name = node.get('{%(text)s}style-name' % self.ns)
         style = self.list_styles[style_name]
+        if style is None:
+            return False
         return style['numbered']
 
     def handle_node(self, node):
